/*
 * $Id: glue-gui-gtk.c,v 1.6 2008-04-30 16:20:29 potyra Exp $
 */

#include <assert.h>
#include <fcntl.h>
#include <stdio.h>

#include <gtk/gtk.h>
#include <glib.h>

#if defined(DARWIN)
#include <sys/types.h>
#endif

#include "glue-gui-gtk.h"
#include "glue-io.h"

static GMainContext *main_context = NULL;
static GPollFD fds_gpoll[FD_SETSIZE];
static gint max_priority;
static int fds_length = 0;
static char gui_fd_registered[FD_SETSIZE];


static void
gui_gtk_io(int fd, void *s)
{
	gui_gtk_flush();
}


static void
gui_gtk_reg(int fd)
{
	int flags;
	int ret;

	io_register(fd, (void *) 1, gui_gtk_io);

	/* remove the following, to make it work on hardy */
	flags = fcntl(fd, F_GETFL, 0);
	assert(0 <= flags);

	flags &= ~O_NONBLOCK;

	ret = fcntl(fd, F_SETFL, flags);
	assert(0 <= ret);
	/* until here */
}

static void
gui_gtk_prepare_event(void)
{
	int ret;
	gint timeout;
	unsigned int i;
	unsigned int j;

	ret =  g_main_context_acquire(main_context);
	assert(ret == TRUE);

	g_main_context_prepare(main_context, &max_priority);

	fds_length = g_main_context_query(main_context,
			max_priority, &timeout, fds_gpoll, FD_SETSIZE);

	for (i = 0; i < FD_SETSIZE; i++) {
		for (j = 0; ; j++) {
			if (j == fds_length) {
				/* Not found. */
				if (gui_fd_registered[i]) {
					io_unregister(i);
					gui_fd_registered[i] = 0;
				}
				break;
			}
			if (fds_gpoll[j].fd == i
			 && (fds_gpoll[j].events & G_IO_IN)) {
				/* Found. */
				if (! gui_fd_registered[i]) {
					gui_gtk_reg(i);
					gui_fd_registered[i] = 1;
				}
				break;
			}
		}
	}

	g_main_context_release(main_context);
}

static int
gui_gtk_handle_event(void)
{
	unsigned int i;
	int ret;

	ret =  g_main_context_acquire(main_context);
	assert(ret == TRUE);

	for (i = 0; i < fds_length; i++) {
		fds_gpoll[i].revents = G_IO_IN;
	}

	if (g_main_context_check(main_context, max_priority,
			fds_gpoll, fds_length)) {
		g_main_context_dispatch(main_context);
		ret = 1;
	} else {
		ret = 0;
	}

	g_main_context_release(main_context);

	return ret;
}

void
gui_gtk_flush(void)
{
	static int running = 0;
	int more;

	if (running) {
		return;
	}
	running = 1;

	more = gui_gtk_handle_event();
	do {
		gdk_display_flush(gdk_display_get_default());
		gui_gtk_prepare_event();
		more = gui_gtk_handle_event();
	} while (more);
	gui_gtk_prepare_event();

	running = 0;
}

void
gui_gtk_init(void)
{
	main_context = g_main_context_default();

	gui_gtk_prepare_event();
}

void
gui_gtk_exit(void)
{
	(void) gui_gtk_handle_event();
}
